from django.conf import settings
from django.db import models
from django.utils import timezone


class Article(models.Model):
    title = models.CharField(max_length=255)
    slug = models.SlugField(unique=True)
    content = models.TextField()
    image = models.ImageField(null=True, blank=True)
    pub_date = models.DateTimeField(default=timezone.now)
    safe_for_work = models.BooleanField(default=False)
    minutes_to_read = models.IntegerField(default=5)
    author = models.ForeignKey(
        settings.AUTH_USER_MODEL, on_delete=models.CASCADE
    )

    def __str__(self):
        return self.title



from datetime import datetime

from tortoise import fields
from tortoise.models import Model


class User(Model):
    id = fields.IntField(pk=True)
    username = fields.CharField(max_length=255)
    first_name = fields.CharField(max_length=255)
    last_name = fields.CharField(max_length=255)
    email = fields.CharField(max_length=255)
    password = fields.CharField(max_length=255, null=True, blank=True)
    last_login = fields.DatetimeField(null=True, blank=True)
    is_superuser = fields.BooleanField(default=False)
    is_staff = fields.BooleanField(default=False)
    is_active = fields.BooleanField(default=True)
    date_joined = fields.DatetimeField(null=True, blank=True)

    def __str__(self):
        return self.title


class Article(Model):
    id = fields.IntField(pk=True)
    title = fields.CharField(max_length=255)
    slug = fields.CharField(max_length=255, unique=True)
    content = fields.TextField()
    image = fields.TextField(null=True, blank=True)
    pub_date = fields.DatetimeField(default=datetime.now)
    safe_for_work = fields.BooleanField(default=False)
    minutes_to_read = fields.IntField(default=5)
    author = fields.ForeignKeyField("models.User", on_delete=fields.CASCADE)

    def __str__(self):
        return self.title



from pathlib import Path

from fake import Factory, FileSystemStorage, SubFactory, TortoiseModelFactory

from article.models import Article, User

BASE_DIR = Path(__file__).resolve().parent.parent
MEDIA_ROOT = BASE_DIR / "media"

STORAGE = FileSystemStorage(root_path=MEDIA_ROOT, rel_path="tmp")


class UserFactory(TortoiseModelFactory):
    id = Factory.pyint()
    username = Factory.username()
    first_name = Factory.first_name()
    last_name = Factory.last_name()
    email = Factory.email()
    last_login = Factory.date_time()
    is_superuser = False
    is_staff = False
    is_active = Factory.pybool()
    date_joined = Factory.date_time()

    class Meta:
        model = User


class ArticleFactory(TortoiseModelFactory):
    id = Factory.pyint()
    title = Factory.sentence()
    slug = Factory.slug()
    content = Factory.text()
    image = Factory.png_file(storage=STORAGE)
    pub_date = Factory.date_time()
    safe_for_work = Factory.pybool()
    minutes_to_read = Factory.pyint(min_value=1, max_value=10)
    author = SubFactory(UserFactory)

    class Meta:
        model = Article



from django.contrib import admin

from article.models import Article


@admin.register(Article)
class ArticleAdmin(admin.ModelAdmin):
    list_display = (
        "title",
        "slug",
        "minutes_to_read",
        "safe_for_work",
        "pub_date",
    )
    list_filter = ("safe_for_work",)



from dataclasses import dataclass
from datetime import date, datetime
from typing import Optional

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = (
    "Article",
    "User",
)


@dataclass
class User:
    id: int
    username: str
    first_name: str
    last_name: str
    email: str
    last_login: Optional[datetime]
    date_joined: Optional[datetime]
    password: Optional[str] = None
    is_superuser: bool = False
    is_staff: bool = False
    is_active: bool = True

    def __str__(self):
        return self.username


@dataclass
class Article:
    id: int
    title: str
    slug: str
    content: str
    headline: str
    category: str
    author: User
    image: Optional[str] = None  # Use str to represent the image path or URL
    pub_date: date = date.today()
    safe_for_work: bool = False
    minutes_to_read: int = 5

    def __str__(self):
        return self.title



import unittest
from datetime import datetime

from fake import FILE_REGISTRY

from article.factories import ArticleFactory
from article.models import Article, User

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("FactoriesTestCase",)


class FactoriesTestCase(unittest.TestCase):
    def tearDown(self):
        FILE_REGISTRY.clean_up()

    def test_sub_factory(self) -> None:
        article = ArticleFactory()

        # Testing SubFactory
        self.assertIsInstance(article.author, User)
        self.assertIsInstance(article.author.id, int)
        self.assertIsInstance(article.author.is_staff, bool)
        self.assertIsInstance(article.author.date_joined, datetime)

        # Testing Factory
        self.assertIsInstance(article.id, int)
        self.assertIsInstance(article.slug, str)

        # Testing hooks
        self.assertTrue(
            hasattr(article, "pre_save_called") and article.pre_save_called
        )
        self.assertTrue(
            hasattr(article, "post_save_called") and article.post_save_called
        )
        self.assertTrue(
            hasattr(article.author, "pre_save_called")
            and article.author.pre_save_called
        )
        self.assertTrue(
            hasattr(article.author, "post_save_called")
            and article.author.post_save_called
        )

        # Testing batch creation
        articles = ArticleFactory.create_batch(5)
        self.assertEqual(len(articles), 5)
        self.assertIsInstance(articles[0], Article)



from datetime import date

from tortoise import fields
from tortoise.models import Model

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = (
    "Article",
    "User",
)


class User(Model):
    """User model."""

    id = fields.IntField(pk=True)
    username = fields.CharField(max_length=255, unique=True)
    first_name = fields.CharField(max_length=255)
    last_name = fields.CharField(max_length=255)
    email = fields.CharField(max_length=255)
    password = fields.CharField(max_length=255, null=True, blank=True)
    last_login = fields.DatetimeField(null=True, blank=True)
    is_superuser = fields.BooleanField(default=False)
    is_staff = fields.BooleanField(default=False)
    is_active = fields.BooleanField(default=True)
    date_joined = fields.DatetimeField(null=True, blank=True)

    def __str__(self):
        return self.title


class Article(Model):
    """Article model."""

    id = fields.IntField(pk=True)
    title = fields.CharField(max_length=255)
    slug = fields.CharField(max_length=255, unique=True)
    content = fields.TextField()
    image = fields.TextField(null=True, blank=True)
    pub_date = fields.DateField(default=date.today)
    safe_for_work = fields.BooleanField(default=False)
    minutes_to_read = fields.IntField(default=5)
    author = fields.ForeignKeyField("models.User", on_delete=fields.CASCADE)

    def __str__(self):
        return self.title



__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = (
    "FIRST_NAMES",
    "LAST_NAMES",
    "WORDS",
    "STREET_NAMES",
    "CITIES",
    "REGIONS",
)

# Custom first names dictionary
FIRST_NAMES = [
    "Anahit",
    "Ani",
    "Aram",
    "Areg",
    "Artur",
    "Astghik",
    "Atom",
    "Barsegh",
    "Gaiane",
    "Gor",
    "Hakob",
    "Hasmik",
    "Levon",
    "Lilit",
    "Mariam",
    "Nare",
    "Narek",
    "Nune",
    "Raffi",
    "Shant",
    "Tatev",
    "Tigran",
    "Vahan",
    "Vardan",
]

# Custom last names dictionary
LAST_NAMES = [
    "Amatouni",
    "Avagyan",
    "Danielyan",
    "Egoyan",
    "Gevorgyan",
    "Gnouni",
    "Grigoryan",
    "Hakobyan",
    "Harutyunyan",
    "Hovhannisyan",
    "Karapetyan",
    "Khachatryan",
    "Manukyan",
    "Ter-Martirosyan",
    "Melikyan",
    "Mkrtchyan",
    "Petrosyan",
    "Saroyan",
    "Sahakyants",
    "Sargsyan",
    "Sedrakyan",
    "Simonyan",
    "Stepanyan",
    "Vardanyan",
]

# Custom words dictionary
WORDS = [
    "time",
    "person",
    "year",
    "way",
    "day",
    "thing",
    "man",
    "world",
    "life",
    "hand",
    "part",
    "child",
    "eye",
    "woman",
    "place",
    "work",
    "week",
    "case",
    "point",
    "government",
    "company",
    "number",
    "group",
    "problem",
    "fact",
    "be",
    "have",
    "do",
    "say",
    "get",
    "make",
    "go",
    "know",
    "take",
    "see",
    "come",
    "think",
    "look",
    "want",
    "give",
    "use",
    "find",
    "tell",
    "ask",
    "work",
    "seem",
    "feel",
    "try",
    "leave",
    "call",
    "good",
    "new",
    "first",
    "last",
    "long",
    "great",
    "little",
    "own",
    "other",
    "old",
    "right",
    "big",
    "high",
    "different",
    "small",
    "large",
    "next",
    "early",
    "young",
    "important",
    "few",
    "public",
    "bad",
    "same",
    "able",
    "to",
    "of",
    "in",
    "for",
    "on",
    "with",
    "as",
    "at",
    "by",
    "from",
    "up",
    "about",
    "into",
    "over",
    "after",
    "beneath",
    "under",
    "above",
    "the",
    "and",
    "a",
    "that",
    "I",
    "it",
    "not",
]

STREET_NAMES = [
    "Oranjestraat",
    "Groningerweg",
    "Prinsengracht",
    "Keizersgracht",
    "Herengracht",
    "Singel",
    "Zuidstraat",
    "Noordstraat",
    "Ooststraat",
    "Weststraat",
    "Nieuweweg",
    "Oudeweg",
    "Parklaan",
    "Beukenweg",
    "Dorpsstraat",
    "Kerkstraat",
    "Molenstraat",
    "Schoolstraat",
    "Stationsstraat",
    "Bloemenstraat",
    "Bosweg",
    "Duinweg",
    "Gracht",
    "Havenstraat",
    "Hoogstraat",
    "Kanaalstraat",
    "Kastanjelaan",
    "Lindelaan",
    "Marktstraat",
    "Meidoornstraat",
    "Plataanstraat",
    "Populierenlaan",
    "Ringweg",
    "Rivierenlaan",
    "Sparrenlaan",
    "Sportlaan",
    "Tuinstraat",
    "Vijverlaan",
    "Wilgenweg",
    "Zonnelaan",
    "Acacialaan",
    "Berkenweg",
    "Cederstraat",
    "Dennenlaan",
    "Esdoornstraat",
    "Hazelaarstraat",
    "Iepenlaan",
    "Jasmijnstraat",
    "Kamperfoeliestraat",
    "Larixlaan",
]

CITIES = [
    "Amsterdam",
    "Rotterdam",
    "Den Haag",
    "Utrecht",
    "Eindhoven",
    "Tilburg",
    "Groningen",
    "Almere",
    "Breda",
    "Nijmegen",
    "Enschede",
    "Haarlem",
    "Arnhem",
    "Zaanstad",
    "Amersfoort",
    "Apeldoorn",
    "Hoofddorp",
    "Maastricht",
    "Leiden",
    "Dordrecht",
    "Zoetermeer",
    "Zwolle",
    "Deventer",
    "Born",
    "Delft",
    "Alkmaar",
    "Heerlen",
    "Venlo",
    "Leeuwarden",
    "Hilversum",
    "Hengelo",
    "Amstelveen",
    "Roosendaal",
    "Purmerend",
    "Oss",
    "Schiedam",
    "Spijkenisse",
    "Helmond",
    "Vlaardingen",
    "Almelo",
    "Gouda",
    "Zaandam",
    "Lelystad",
    "Alphen aan den Rijn",
    "Hoorn",
    "Velsen",
    "Ede",
    "Bergen op Zoom",
    "Capelle aan den IJssel",
    "Assen",
]

REGIONS = [
    "Drenthe",
    "Flevoland",
    "Friesland",
    "Gelderland",
    "Groningen",
    "Limburg",
    "Noord-Brabant",
    "Noord-Holland",
    "Overijssel",
    "Utrecht",
    "Zeeland",
    "Zuid-Holland",
]



from dataclasses import dataclass

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("Band",)


@dataclass
class Band:
    id: int
    name: str

    def __str__(self) -> str:
        return self.address_line



from datetime import date, datetime
from typing import Optional

from pydantic import BaseModel, Field

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = (
    "Article",
    "User",
)


class User(BaseModel):
    id: int
    username: str = Field(..., max_length=255)
    first_name: str = Field(..., max_length=255)
    last_name: str = Field(..., max_length=255)
    email: str = Field(..., max_length=255)
    password: Optional[str] = Field("", max_length=255)
    last_login: Optional[datetime] = None
    is_superuser: bool = Field(default=False)
    is_staff: bool = Field(default=False)
    is_active: bool = Field(default=True)
    date_joined: Optional[datetime] = None

    class Config:
        extra = "allow"  # For testing purposes only

    def __str__(self):
        return self.username


class Article(BaseModel):
    id: int
    title: str = Field(..., max_length=255)
    slug: str = Field(..., max_length=255, unique=True)
    content: str
    image: Optional[str] = None  # Use str to represent the image path or URL
    pub_date: date = Field(default_factory=date.today)
    safe_for_work: bool = False
    minutes_to_read: int = 5
    author: User

    class Config:
        extra = "allow"  # For testing purposes only

    def __str__(self):
        return self.title



from datetime import datetime

from fake import FILE_REGISTRY
from tortoise.contrib import test
from tortoise.contrib.test import finalizer, initializer

from article.factories import ArticleFactory
from article.models import Article, User

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("FactoriesTestCase",)


class FactoriesTestCase(test.TestCase):
    @classmethod
    def setUpClass(cls):
        initializer(["article.models"], db_url="sqlite://:memory:")
        # Your setup code...

    @classmethod
    def tearDownClass(cls):
        finalizer()
        # Your teardown code...

    def tearDown(self):
        FILE_REGISTRY.clean_up()

    def test_sub_factory(self) -> None:
        with self.subTest("SubFactory"):
            article = ArticleFactory()

            # Testing SubFactory
            self.assertIsInstance(article.author, User)
            self.assertIsInstance(article.author.id, int)
            self.assertIsInstance(article.author.is_staff, bool)
            self.assertIsInstance(article.author.date_joined, datetime)

            # Testing Factory
            self.assertIsInstance(article.id, int)
            self.assertIsInstance(article.slug, str)

            # Testing hooks
            self.assertTrue(
                hasattr(article, "pre_save_called") and article.pre_save_called
            )
            self.assertTrue(
                hasattr(article, "post_save_called")
                and article.post_save_called
            )
            self.assertTrue(
                hasattr(article.author, "pre_save_called")
                and article.author.pre_save_called
            )
            self.assertTrue(
                hasattr(article.author, "post_save_called")
                and article.author.post_save_called
            )

            # Testing batch creation
            articles = ArticleFactory.create_batch(5)
            self.assertEqual(len(articles), 5)
            self.assertIsInstance(articles[0], Article)

        with self.subTest("SubFactory nested params"):
            article = ArticleFactory(author__username="admin")

            # Testing SubFactory
            self.assertIsInstance(article.author, User)
            self.assertIsInstance(article.author.id, int)
            self.assertIsInstance(article.author.is_staff, bool)
            self.assertIsInstance(article.author.date_joined, datetime)
            self.assertEqual(article.author.username, "admin")

            # Testing Factory
            self.assertIsInstance(article.id, int)
            self.assertIsInstance(article.slug, str)

            # Testing hooks
            self.assertTrue(
                hasattr(article, "pre_save_called") and article.pre_save_called
            )
            self.assertTrue(
                hasattr(article, "post_save_called")
                and article.post_save_called
            )
            self.assertTrue(
                hasattr(article.author, "pre_save_called")
                and article.author.pre_save_called
            )
            self.assertTrue(
                hasattr(article.author, "post_save_called")
                and article.author.post_save_called
            )

            # Testing batch creation
            articles = ArticleFactory.create_batch(5)
            self.assertEqual(len(articles), 5)
            self.assertIsInstance(articles[0], Article)



from datetime import datetime
from typing import Optional

from pydantic import BaseModel, Field


class User(BaseModel):
    id: int
    username: str = Field(..., max_length=255)
    first_name: str = Field(..., max_length=255)
    last_name: str = Field(..., max_length=255)
    email: str = Field(..., max_length=255)
    password: Optional[str] = Field("", max_length=255)
    last_login: Optional[datetime]
    is_superuser: bool = Field(default=False)
    is_staff: bool = Field(default=False)
    is_active: bool = Field(default=True)
    date_joined: Optional[datetime]

    def __str__(self):
        return self.username


class Article(BaseModel):
    id: int
    title: str = Field(..., max_length=255)
    slug: str = Field(..., max_length=255, unique=True)
    content: str
    image: Optional[str] = None  # Use str to represent the image path or URL
    pub_date: datetime = Field(default_factory=datetime.now)
    safe_for_work: bool = False
    minutes_to_read: int = 5
    author: User

    def __str__(self):
        return self.title



from dataclasses import dataclass
from datetime import date

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = (
    "Address",
    "Person",
)


@dataclass
class Address:
    id: int
    address_line: str
    postal_code: str
    city: str
    region: str

    def __str__(self) -> str:
        return self.address_line


@dataclass
class Person:
    id: int
    first_name: str
    last_name: str
    email: str
    dob: date
    address: Address

    def __str__(self) -> str:
        return self.username



from datetime import datetime

from sqlalchemy import (
    Boolean,
    Column,
    DateTime,
    ForeignKey,
    Integer,
    String,
    Text,
)
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relationship

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = (
    "Article",
    "Base",
    "User",
)


Base = declarative_base()


class User(Base):
    """User model."""

    __tablename__ = "users"

    id = Column(Integer, primary_key=True)
    username = Column(String(255), unique=True)
    first_name = Column(String(255))
    last_name = Column(String(255))
    email = Column(String(255))
    password = Column(String(255), nullable=True)
    last_login = Column(DateTime, nullable=True)
    is_superuser = Column(Boolean, default=False)
    is_staff = Column(Boolean, default=False)
    is_active = Column(Boolean, default=True)
    date_joined = Column(DateTime, nullable=True)

    articles = relationship("Article", back_populates="author")

    def __repr__(self):
        return f"<User(username='{self.username}', email='{self.email}')>"


class Article(Base):
    """Article model."""

    __tablename__ = "articles"

    id = Column(Integer, primary_key=True)
    title = Column(String(255))
    slug = Column(String(255), unique=True)
    content = Column(Text)
    image = Column(Text, nullable=True)
    pub_date = Column(DateTime, default=datetime.utcnow)
    safe_for_work = Column(Boolean, default=False)
    minutes_to_read = Column(Integer, default=5)
    author_id = Column(Integer, ForeignKey("users.id"))

    author = relationship("User", back_populates="articles")

    def __repr__(self):
        return (
            f"<Article(title='{self.title}', author='{self.author.username}')>"
        )



import unittest

from band.factories import BandFactory
from band.models import Band
from fake import FILE_REGISTRY

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("BandFakerTestCase",)


class BandFakerTestCase(unittest.TestCase):
    def tearDown(self):
        FILE_REGISTRY.clean_up()

    def test_sub_factory(self) -> None:
        band = BandFactory()

        # Testing Factory
        self.assertIsInstance(band.id, int)
        self.assertIsInstance(band.name, str)

        # Testing batch creation
        bands = BandFactory.create_batch(5)
        self.assertEqual(len(bands), 5)
        self.assertIsInstance(bands[0], Band)



from django.conf import settings
from django.contrib.auth.models import Group, User
from django.utils import timezone
from fake import (
    FACTORY,
    DjangoModelFactory,
    FileSystemStorage,
    PostSave,
    PreSave,
    SubFactory,
    post_save,
    pre_save,
    trait,
)

from article.models import Article

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = (
    "ArticleFactory",
    "UserFactory",
)

STORAGE = FileSystemStorage(root_path=settings.MEDIA_ROOT, rel_path="tmp")


class GroupFactory(DjangoModelFactory):
    """Group factory."""

    name = FACTORY.word()

    class Meta:
        model = Group
        get_or_create = ("name",)


def set_password(user: User, password: str) -> None:
    user.set_password(password)


def add_to_group(user: User, group_name: str) -> None:
    group = GroupFactory(name=group_name)
    user.groups.add(group)


class UserFactory(DjangoModelFactory):
    """User factory.

    Usage example:

    .. code-block:: python

        user = UserFactory()
        users = UserFactory.create_batch(5)
    """

    username = FACTORY.username()
    first_name = FACTORY.first_name()
    last_name = FACTORY.last_name()
    email = FACTORY.email()
    last_login = FACTORY.date_time(tzinfo=timezone.get_current_timezone())
    is_superuser = False
    is_staff = False
    is_active = FACTORY.pybool()
    date_joined = FACTORY.date_time(tzinfo=timezone.get_current_timezone())
    password = PreSave(set_password, password="test1234")
    group = PostSave(add_to_group, group_name="TestGroup1234")

    class Meta:
        model = User
        get_or_create = ("username",)

    @trait
    def is_admin_user(self, instance: User) -> None:
        instance.is_superuser = True
        instance.is_staff = True
        instance.is_active = True

    @pre_save
    def _pre_save_method(self, instance):
        instance.pre_save_called = True

    @post_save
    def _post_save_method(self, instance):
        instance.post_save_called = True


class ArticleFactory(DjangoModelFactory):
    """Article factory.

    Usage example:

        article = ArticleFactory()
        articles = ArticleFactory.create_batch(5)
    """

    title = FACTORY.sentence()
    slug = FACTORY.slug()
    content = FACTORY.text()
    image = FACTORY.png_file(storage=STORAGE)
    pub_date = FACTORY.date(tzinfo=timezone.get_current_timezone())
    safe_for_work = FACTORY.pybool()
    minutes_to_read = FACTORY.pyint(min_value=1, max_value=10)
    author = SubFactory(UserFactory)

    class Meta:
        model = Article

    @pre_save
    def _pre_save_method(self, instance):
        instance.pre_save_called = True

    @post_save
    def _post_save_method(self, instance):
        instance.post_save_called = True



import random

from fake import Factory, Faker, provider

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = (
    "FACTORY",
    "FAKER",
)

BANDS = [
    "Aerosmith",
    "Agata Kristi",
    "Anathema",
    "Bjork",
    "Black Sabbath",
    "Blonde Redhead",
    "Cocteau Twins",
    "Comus",
    "Dead Can Dance",
    "Depeche Mode",
    "Focus",
    "Kalinov Most",
    "Kate Bush",
    "King Crimson",
    "Kolibri",
    "Lais",
    "Led Zeppelin",
    "Linda",
    "Nirvana",
    "Nor Dar",
    "Oasis",
    "Opeth",
    "Pink Floyd",
    "Portishead",
    "Radiohead",
    "Red Hot Chili Peppers",
    "Sinead O'Connor",
    "Siouxsie and the Banshees",
    "System of a Down",
    "The Beatles",
    "The Cranberries",
    "The Cure",
    "The Rolling Stones",
    "The Velvet Underground",
    "Tiamat",
    "Van der Graaf Generator",
    "Yes",
]


class FakerBand(Faker):
    """Custom Faker class for bands."""

    @provider
    def band_name(self) -> str:
        return random.choice(BANDS)


FAKER = FakerBand(alias="band")
FACTORY = Factory(FAKER)



from pathlib import Path

from fake import (
    FACTORY,
    FileSystemStorage,
    ModelFactory,
    PostSave,
    PreSave,
    SubFactory,
    post_save,
    pre_save,
    trait,
)

from article.models import Article, Group, User

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = (
    "ArticleFactory",
    "UserFactory",
)

# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent.parent
MEDIA_ROOT = BASE_DIR / "media"

STORAGE = FileSystemStorage(root_path=MEDIA_ROOT, rel_path="tmp")


class GroupFactory(ModelFactory):
    id = FACTORY.pyint()
    name = FACTORY.word()

    class Meta:
        model = Group
        get_or_create = ("name",)


def set_password(user: User, password: str) -> None:
    user.set_password(password)


def add_to_group(user: User, name: str) -> None:
    group = GroupFactory(name=name)
    user.groups.add(group)


class UserFactory(ModelFactory):
    id = FACTORY.pyint()  # type: ignore
    username = FACTORY.username()  # type: ignore
    first_name = FACTORY.first_name()  # type: ignore
    last_name = FACTORY.last_name()  # type: ignore
    email = FACTORY.email()  # type: ignore
    last_login = FACTORY.date_time()  # type: ignore
    is_superuser = False
    is_staff = False
    is_active = FACTORY.pybool()  # type: ignore
    date_joined = FACTORY.date_time()  # type: ignore
    password = PreSave(set_password, password="test1234")  # type: ignore
    group = PostSave(add_to_group, name="TestGroup1234")  # type: ignore

    class Meta:
        model = User

    @trait
    def is_admin_user(self, instance: User) -> None:
        instance.is_superuser = True
        instance.is_staff = True
        instance.is_active = True

    @pre_save
    def _pre_save_method(self, instance):
        instance.pre_save_called = True

    @post_save
    def _post_save_method(self, instance):
        instance.post_save_called = True


class ArticleFactory(ModelFactory):
    id = FACTORY.pyint()  # type: ignore
    title = FACTORY.sentence()  # type: ignore
    slug = FACTORY.slug()  # type: ignore
    content = FACTORY.text()  # type: ignore
    image = FACTORY.png_file(storage=STORAGE)  # type: ignore
    pub_date = FACTORY.date()  # type: ignore
    safe_for_work = FACTORY.pybool()  # type: ignore
    minutes_to_read = FACTORY.pyint(min_value=1, max_value=10)  # type: ignore
    author = SubFactory(UserFactory)

    class Meta:
        model = Article

    @pre_save
    def _pre_save_method(self, instance):
        instance.pre_save_called = True

    @post_save
    def _post_save_method(self, instance):
        instance.post_save_called = True
